/***************************************************************
 * Name:      wxiscanframe.h
 * Purpose:   Defines Application Frame
 * Author:    Daniel Nell (daniel.nell@nellresearch.de)
 * Created:   2008-05-04
 * Copyright: Daniel Nell (www.nellresearch.de)
 * License:   GPL
 **************************************************************/

#ifndef WXISCANMAIN_H
#define WXISCANMAIN_H

//(*Headers(wxIScanFrame)
#include <wx/toolbar.h>
#include <wx/menu.h>
#include <wx/scrolwin.h>
#include <wx/splitter.h>
#include <wx/statusbr.h>
#include <wx/frame.h>
#include <wx/textctrl.h>
//*)

// Additional wxWidgets header files
#include <wx/docview.h>
#include <wx/process.h>

// Private header files.
#include "wxextfiledroptarget.h"
#include "wxiviewcanvas.h"
#include "wxiviewpaint.h"
#if __WXLUASCRIPT__
#   include "wxluascript.h"
#endif // WXLUASCRIPT__


// IDs
#define wxISCAN_PROCESS     1


//////////////////////////////////////////////////////////
// Predefined classes
//
#if __WXPDFDOC__
class wxPdfDocument;
#endif // __WXPDFDOC__
class wxExtProcess;
#if __WXLUASCRIPT__
struct lua_State;
#endif // WXLUASCRIPT__


#if __WXLUASCRIPT__
/////////////////////////////////////////////////////////////////////////////
//  Class wxIScanLuaScript
//
/// \brief An extended class to handle Lua scripting.
///
/// ...
class wxIScanLuaScript : public wxLuaScript
{
    public:
        /// \brief Standard constructor.
        ///
        /// \param strScript     ...
        /// \param wxEvtHandler  A pointer to the parent event handler (mostly the parent window).
        /// \param pbParentState ...
        wxIScanLuaScript( const wxString& strScript= wxEmptyString, wxEvtHandler *poParent= NULL, bool *pbParentState= NULL );

    private:
        // Addtional Lua functions.
        /// \brief Do OCR on the loaded/scanned image and return the text (to Lua).
        ///
        /// \param L      a pointer to the Lua context.
        static int OCR( lua_State* L );

        /// \brief Open an image file in Lua using a given name.
        ///
        /// \param L      a pointer to the Lua context.
        static int OpenImage( lua_State* L );

        /// \brief Save an image file in Lua using a given name.
        ///
        /// \param L      a pointer to the Lua context.
        static int SaveImage( lua_State* L );

        /// \brief Copy a given text to the clipboard in Lua.
        ///
        /// \param L      a pointer to the Lua context.
        static int CopyText2Clipboard( lua_State* L );

        /// \brief Replace the program's image by its monochrome version.
        ///
        /// \param L      a pointer to the Lua context.
        static int Image2Mono( lua_State* L );

        /// \brief Replace the program's image by it version that is colour enhenced.
        ///
        /// \param L      a pointer to theLua context.
        static int EnhenceColours( lua_State* L );

        /// \brief Replace white colour by yellow.
        ///
        /// \param L      a pointer to the Lua context.
        static int EnhenceOpticalContrast( lua_State* L );

        /// \brief Fork an external process.
        ///
        /// \param L      a pointer to the Lua context.
        static int Execute( lua_State* L );
};
#endif // __WXLUASCRIPT__


//////////////////////////////////////////////////////////
// Class wxIScanFrame
//
/// \brief The frame window class.
///
/// ...
class wxIScanFrame: public wxFrame, public wxIViewPaintBase
{
    public:
        enum
        {
            wxCANVASFULLSIZE,                                       ///< Show image full size (without any zooming).
            wxCANVASFITWIDTH,                                       ///< Zoom image to fit the window width.
            wxCANVASFITWINDOW,                                      ///< Zoom image to fit the window size.
            wxCANVASCUSTOMSIZE                                      ///< Zoom image to an explicit given zoom factor (m_nScaleFactor).
        };

    public:
        /// \brief Standard constructor.
        ///
        /// \param poParent     a pointer to the parent window
        /// \param nId          the window id
        wxIScanFrame( wxWindow* poParent, wxWindowID nId= wxID_ANY );

        /// \brief Virtual destructor.
        virtual ~wxIScanFrame();

        virtual bool CanClose( bool bCanVeto= true );               ///< Handle window closing requests.
        virtual bool OpenImage( const wxString &strFileName );      ///< Open an image using the given name.
        virtual bool OnDocumentPageChanged();                       ///< ...
        virtual void SetBitmap();                                   ///< ...
        virtual wxImage GetImageSelection();                        ///< ...
        virtual bool ScanImage();                                   ///< ...
        virtual void SetProfile( int nProfileIndex );               ///< ...
        virtual void ApplyProfile();                                ///< ...

        /// \brief Enhance the optical contrast by replacing white by yellow.
        virtual void EnhenceOpticalContrast()
        {
            m_oImage.Replace( 255, 255, 255, 255, 255, 0 );
            SetBitmap();
        }

        virtual wxString OCR( const wxString &strTempFileName );    ///< ...
        virtual bool ReadOut( bool bAsync= true );                  ///< Do OCR and read out the returned text.

        /// \brief Fork an external process.
        ///
        /// \param strCommand           the command line to be executed.
        /// \param bAsync               an (optional) flag which decides if the new process
        ///                             is to be executed synchronous (default) or asynchronous.
        /// \param strTempTextFileName  an (optional) file name of a file that should be deleted
        ///                             at the end of an asynchronous running process.
        virtual bool Execute( const wxString &strCommand, bool bAsync= false,
                              const wxString &strTempTextFileName= wxEmptyString );

#if __WXPDFDOC__
        virtual bool CreateNewPdfDocument();                        ///< Create (and initialize) a new PDF document.
        virtual bool AddPdfPage();                                  ///< Add a new image/OCR text page to the existing PDF document.
        virtual bool IsPdfDirty(){ return m_bPdfDirty; }            ///< ...
#endif // __WXPDFDOC__
        virtual wxString GetTextFromFile( const wxString &strFilename ); ///< ...
        virtual bool CopyText2Clipboard( const wxString &strText ); ///< Copy the given text to the sytem clipboard.
        virtual bool CopyImage2Clipboard( const wxImage &oImage );  ///< Copy the given image to the sytem clipboard.
        virtual void ResetFileName();                               ///< ...
        virtual void RestoreSettings();                             ///< ...
        virtual void SaveSettings();                                ///< ...
        virtual void EnableControls();                              ///< ...
        virtual void LockUI( bool bLock= true ){ m_bUILocked= bLock; EnableControls(); } ///< ...
        virtual void UnlockUI(){ LockUI( false ); }                 ///< ...

    public:
        // Printer handling (introduced by base class wxIViewPaintBase)
        virtual void AdjustOrientation(){ m_oPrintData.SetOrientation( m_bPortrait ? wxPORTRAIT : wxLANDSCAPE ); }
        virtual bool GetPageSizeMM( int *pnWidth, int *pnHeight );
        virtual int  GetMaxPage(){ return m_nImageCount; }
        virtual bool GotoPage( int nPage );
        virtual bool Paint( wxDC& oDc );

    private:
        // Member variables
        bool m_bUILocked;                                           ///< States if the GUI should be locked by wxIScanFrame::EnableControls().
        wxImage m_oImage;                                           ///< Image object.
        wxImage m_oPrintImage;                                      ///< Image object for printing.
        wxString m_strImageFileName;                                ///< Image file name.
        int m_nPage;                                                ///< ...
        int m_nImageCount;                                          ///< ...
        int m_nResolution;                                          ///< ...
        int m_bPortrait;                                            ///< ...

        // Options
        int m_nFit;                                                 ///< ...
        double m_nScaleFactor;                                      ///< Zoom factor.
        bool m_bAutoApplyProfile;                                   ///< ...
        int m_nProfileIndex;                                        ///< ...
        wxString m_strPdfBitmapMimeType;                            ///< Image mimetype for pdf export.
        bool m_bOCR;                                                ///< ...
        bool m_bCmdConvertEnable;                                   ///< ...
        wxString m_strCmdConvert;                                   ///< ...
        bool m_bCmdOcrEnable;                                       ///< ...
        wxString m_strCmdOcr;                                       ///< ...
        bool m_bCmdCleanUpEnable;                                   ///< ...
        wxString m_strCmdCleanUp;                                   ///< ...
        bool m_bCmdTtsEnable;                                       ///< ...
        wxString m_strCmdTts;                                       ///< ...
        bool m_bCmdEditEnable;                                      ///< ...
        wxString m_strCmdEdit;                                      ///< ...

        // File history
        wxFileHistory m_oFileHistory;                               ///< History of last opened files

        // Printing support.
        wxPrintData m_oPrintData;                                   ///< ...

#if __WXPDFDOC__
        // PDF support
        wxPdfDocument *m_poPdfDoc;                                  ///< PDF Document.
        int m_nPdfPages;                                            ///< Number of PDF pages in memory.
        bool m_bPdfDirty;                                           ///< States if the PDF file is dirty (and should be saved).
#endif // __WXPDFDOC__

#if __WXLUASCRIPT__
        bool bLuaReturnFromEventHandler;                            ///< Lua return value of the event handlers.
#endif // WXLUASCRIPT__

        // ...
        wxExtProcess *m_poRunningProcess;                           ///< A concurrent running process.

    private:
        //(*Handlers(wxIScanFrame)
        void OnFileQuit(wxCommandEvent& event);
        void OnFileScanImage(wxCommandEvent& event);
        void OnFileOpen(wxCommandEvent& event);
        void OnFileSaveAs(wxCommandEvent& event);
        void OnFilePrint( wxCommandEvent &oEvent );             ///< Handle menu command "File|Print".
        void OnFilePrintPreview( wxCommandEvent &oEvent );      ///< Handle menu command "File|Print Preview".
        void OnFilePrintSetup( wxCommandEvent &oEvent );        ///< Handle menu command "File|Page Setup ".
        void OnEditCopy(wxCommandEvent& event);
        void OnPdfBeginPdf(wxCommandEvent& event);
        void OnDocumentCopyText(wxCommandEvent& event);
        void OnDocumentReadText(wxCommandEvent& event);
        void OnDocumentFirstPage(wxCommandEvent& event);
        void OnDocumentPreviousPage(wxCommandEvent& event);
        void OnDocumentNextPage(wxCommandEvent& event);
        void OnDocumentLastPage(wxCommandEvent& event);
        void OnDocumentRotate90Left(wxCommandEvent& event);
        void OnDocumentRotate90Right(wxCommandEvent& event);
        void OnDocumentApplyProfile(wxCommandEvent& event);
        void OnPdfAddPage(wxCommandEvent& event);
        void OnPdfSavePdf(wxCommandEvent& event);
        void OnHelpAbout(wxCommandEvent& event);
        void OnCmdStop(wxCommandEvent& event);
        void OnClose(wxCloseEvent& event);
        void OnEditPaste(wxCommandEvent& event);
        void OnEditSettings(wxCommandEvent& event);
        void OnEditSettingsFile(wxCommandEvent& event);
        void OnDocumentCrop(wxCommandEvent& event);
        void OnEditClearHistorySelected(wxCommandEvent& event);
        void OnEditClearLog(wxCommandEvent& event);
        //*)
        void OnDocumentProfilesChooseProfile( wxCommandEvent& oEvent ); ///< Handle menu command "Document|Profile|[PROFILE #X]".
#if __WXLUASCRIPT__
        void OnScriptStartScript( wxCommandEvent& oEvent );         ///< Handle menu command "Scrips|[SCRIPT NAME]".
#endif // WXLUASCRIPT__
        void OnMRUFiles( wxCommandEvent& oEvent );                  ///< Handle menu command "File|[MOST RECENTLY USED FILE #X]".
        void OnFileDrop( wxExtFileDropEvent& oEvent );              ///< Handle file dropping.
        void OnCanvasSize( wxIViewCanvasSizeEvent& oEvent );        ///< Handle bitmap resizing introduced by canvas resizing.
        void OnProcessTerminated( wxProcessEvent& oEvent );         ///< Handle process termination.

        //(*Identifiers(wxIScanFrame)
        static const long ID_CANVAS;
        static const long ID_LOGTEXTCTRL;
        static const long ID_HSPLITTERWINDOW;
        static const long ID_FILESCANIMAGE;
        static const long ID_FILEOPEN;
        static const long ID_FILESAVEAS;
        static const long ID_FILEPRINT;
        static const long ID_FILEPRINTPREVIEW;
        static const long ID_FILEPRINTSETUP;
        static const long ID_FILEQUIT;
        static const long ID_EDITCOPY;
        static const long ID_EDITPASTE;
        static const long ID_EDITSETTINGS;
        static const long ID_EDITSETTINGSFILE;
        static const long ID_EDITCLEARLOG;
        static const long ID_EDITCLEARHISTORY;
        static const long ID_DOCUMENTCOPYTEXT;
        static const long ID_DOCUMENTREADTEXT;
        static const long ID_CMDSTOP;
        static const long ID_DOCUMENTFIRSTPAGE;
        static const long ID_DOCUMENTPREVIOUSPAGE;
        static const long ID_DOCUMENTNEXTPAGE;
        static const long ID_DOCUMENTLASTPAGE;
        static const long ID_DOCUMENTCROP;
        static const long ID_DOCUMENTROTATE90LEFT;
        static const long ID_DOCUMENTROTATE90RIGHT;
        static const long ID_DOCUMENTAPPLYPROFILE;
        static const long ID_MENUDOCUMENTPROFILES;
        static const long ID_PDFBEGINPDF;
        static const long ID_PDFADDPAGE;
        static const long ID_PDFSAVEASPDF;
        static const long ID_HELPABOUT;
        static const long ID_STATUSBAR;
        static const long ID_FILESCANIMAGE_TB;
        static const long ID_FILEOPEN_TB;
        static const long ID_FILESAVEAS_TB;
        static const long ID_FILEPRINT_TB;
        static const long ID_PDFBEGINPDF_TB;
        static const long ID_PDFADDPAGE_TB;
        static const long ID_PDFSAVEASPDF_TB;
        static const long ID_DOCUMENTFIRSTPAGE_TB;
        static const long ID_DOCUMENTPREVIOUSPAGE_TB;
        static const long ID_DOCUMENTNEXTPAGE_TB;
        static const long ID_DOCUMENTLASTPAGE_TB;
        static const long ID_DOCUMENTREADTEXT_TB;
        static const long ID_CMDSTOP_TB;
        static const long ID_TOOLBAR;
        //*)
        static const long ID_DOCUMENTREADTEXTSYNC;
        static const long ID_DOCUMENTPROFILE0;


        //(*Declarations(wxIScanFrame)
        wxIViewCanvas* m_poCanvas;
        wxToolBarToolBase* ToolBarItem5;
        wxMenu* Menu1;
        wxToolBarToolBase* ToolBarItem11;
        wxToolBarToolBase* ToolBarItem6;
        wxToolBarToolBase* ToolBarItem12;
        wxStatusBar* m_poStatusBar;
        wxToolBarToolBase* ToolBarItem7;
        wxToolBarToolBase* ToolBarItem2;
        wxToolBarToolBase* ToolBarItem9;
        wxToolBarToolBase* ToolBarItem10;
        wxToolBarToolBase* ToolBarItem4;
        wxToolBarToolBase* ToolBarItem1;
        wxToolBarToolBase* ToolBarItem3;
        wxSplitterWindow* m_poHSplitterWindow;
        wxTextCtrl* m_poLogTextCtrl;
        wxToolBarToolBase* ToolBarItem8;
        //*)

        // Friend classes.
#if __WXLUASCRIPT__
        friend class wxIScanLuaScript;
#endif // WXLUASCRIPT__

        // Event table.
        DECLARE_EVENT_TABLE()
};

#endif // WXISCANMAIN_H
